@@ -14,6 +14,11 @@
 #' of observations to be used.
 #' @param clusters An optional bare (unquoted) name of the variable that
 #' corresponds to the clusters in the data.
+#' @param fixed_effects An optional right-sided formula containing the fixed
+#' effects that will be projected out of the data, such as \code{~ blockID}. Do not
+#' pass multiple-fixed effects with intersecting groups. Speed gains are greatest for
+#' variables with large numbers of groups and when using "HC1" or "stata" standard errors.
+#' See 'Details'.
 #' @param se_type The sort of standard error sought. If `clusters` is
 #' not specified the options are "HC0", "HC1" (or "stata", the equivalent),
 #'  "HC2" (default), "HC3", or
@@ -48,7 +53,17 @@
 #' endogenous regressors, and the residuals used are the difference
 #' between the outcome and a fit produced by the second-stage coefficients and the
 #' first-stage (endogenous) regressors. More notes on this can be found at
-#' \href{http://estimatr.declaredesign.org/articles/mathematical-notes.html}{the mathematical appendix}.
+#' \href{https://declaredesign.org/R/estimatr/articles/mathematical-notes.html}{the mathematical appendix}.
+#'
+#' If \code{fixed_effects} are specified, both the outcome, regressors, and instruments
+#' are centered using the method of alternating projections (Halperin 1962; Gaure 2013). Specifying
+#' fixed effects in this way will result in large speed gains with standard error
+#' estimators that do not need to invert the matrix of fixed effects. This means using
+#' "classical", "HC0", "HC1", "CR0", or "stata" standard errors will be faster than other
+#' standard error estimators. Be wary when specifying fixed effects that may result
+#' in perfect fits for some observations or if there are intersecting groups across
+#' multiple fixed effect variables (e.g. if you specify both "year" and "country" fixed effects
+#' with an unbalanced panel where one year you only have data for one country).
 #'
 #' @return An object of class \code{"iv_robust"}.
 #'
@@ -74,12 +89,19 @@
 #'   \item{k}{the number of columns in the design matrix (includes linearly dependent columns!)}
 #'   \item{rank}{the rank of the fitted model}
 #'   \item{vcov}{the fitted variance covariance matrix}
-#'   \item{r.squared}{the \eqn{R^2} of the second stage regrssion}
+#'   \item{r.squared}{the \eqn{R^2} of the second stage regression}
 #'   \item{adj.r.squared}{the \eqn{R^2} of the second stage regression, but penalized for having more parameters, \code{rank}}
 #'   \item{fstatistic}{a vector with the value of the second stage F-statistic with the numerator and denominator degrees of freedom}
 #'   \item{weighted}{whether or not weights were applied}
 #'   \item{call}{the original function call}
-#' We also return \code{terms} with the second stage terms and \code{terms_regressors} with the first stage terms, both of which used by \code{predict}.
+#'   \item{fitted.values}{the matrix of predicted means}
+#' We also return \code{terms} with the second stage terms and \code{terms_regressors} with the first stage terms, both of which used by \code{predict}. If \code{fixed_effects} are specified, then we return \code{proj_fstatistic}, \code{proj_r.squared}, and \code{proj_adj.r.squared}, which are model fit statistics that are computed on the projected model (after demeaning the fixed effects).
+#'
+#' @references
+#'
+#' Gaure, Simon. 2013. "OLS with multiple high dimensional category variables." Computational Statistics & Data Analysis 66: 8-1. \url{http://dx.doi.org/10.1016/j.csda.2013.03.024}
+#'
+#' Halperin, I. 1962. "The product of projection operators." Acta Scientiarum Mathematicarum (Szeged) 23(1-2): 96-99.
 #'
 #' @examples
 #' library(fabricatr)
@@ -88,7 +110,8 @@
 #'   Y = rpois(N, lambda = 4),
 #'   Z = rbinom(N, 1, prob = 0.4),
 #'   D  = Z * rbinom(N, 1, prob = 0.8),
-#'   X = rnorm(N)
+#'   X = rnorm(N),
+#'   G = sample(letters[1:4], N, replace = TRUE)
 #' )
 #'
 #' # Instrument for treatment `D` with encouragement `Z`
@@ -104,6 +127,9 @@
 #' # Again, easy to replicate Stata (again with `small` correction in Stata)
 #' tidy(iv_robust(Y ~ D | Z, data = dat, clusters = cl, se_type = "stata"))
 #'
+#' # We can also specify fixed effects, that will be taken as exogenous regressors
+#' # Speed gains with fixed effects are greatests with "stata" or "HC1" std.errors
+#' tidy(iv_robust(Y ~ D | Z, data = dat, fixed_effects = ~ G, se_type = "HC1"))
 #'
 #' @export
 iv_robust <- function(formula,
@@ -111,6 +137,7 @@ iv_robust <- function(formula,
                       weights,
                       subset,
                       clusters,
+                      fixed_effects,
                       se_type = NULL,
                       ci = TRUE,
                       alpha = .05,
@@ -120,7 +147,8 @@ iv_robust <- function(formula,
     formula = formula,
     weights = weights,
     subset = subset,
-    cluster = clusters
+    cluster = clusters,
+    fixed_effects = fixed_effects
   )
   data <- enquo(data)
   model_data <- clean_model_data(data = data, datargs, estimator = "iv")
@@ -129,6 +157,16 @@ iv_robust <- function(formula,
     warning("More regressors than instruments")
   }
 
+  fes <- is.character(model_data[["fixed_effects"]])
+  if (fes) {
+    yoriginal <- model_data[["outcome"]]
+    Xoriginal <- model_data[["design_matrix"]]
+    model_data <- demean_fes(model_data)
+  } else {
+    yoriginal <- NULL
+    Xoriginal <- NULL
+  }
+
   # -----------
   # First stage
   # -----------
@@ -139,34 +177,41 @@ iv_robust <- function(formula,
       X = model_data$instrument_matrix,
       weights = model_data$weights,
       cluster = model_data$cluster,
+      fixed_effects = model_data$fixed_effects,
       ci = FALSE,
       se_type = "none",
       has_int = attr(model_data$terms, "intercept"),
       alpha = alpha,
       return_fit = TRUE,
-      return_unweighted_fit = TRUE,
       return_vcov = return_vcov,
-      try_cholesky = try_cholesky
+      try_cholesky = try_cholesky,
+      iv_stage = list(1)
     )
 
   # ------
   # Second stage
   # ------
   colnames(first_stage$fitted.values) <- colnames(model_data$design_matrix)
+  if (!is.null(model_data$fixed_effects)) {
+    attr(model_data$fixed_effects, "fe_rank") <- sum(model_data[["fe_levels"]]) + 1
+  }
 
   second_stage <-
     lm_robust_fit(
       y = model_data$outcome,
       X = first_stage$fitted.values,
+      yoriginal = yoriginal,
+      Xoriginal = Xoriginal,
       weights = model_data$weights,
       cluster = model_data$cluster,
+      fixed_effects = model_data$fixed_effects,
       ci = ci,
       se_type = se_type,
       has_int = attr(model_data$terms, "intercept"),
       alpha = alpha,
       return_vcov = return_vcov,
       try_cholesky = try_cholesky,
-      X_first_stage = model_data$design_matrix
+      iv_stage = list(2, model_data$design_matrix)
     )
 
   return_list <- lm_return(
