@@ -25,15 +25,8 @@ test_that("iv_robust warnings and errors are correct", {
 
 test_that("iv_robust matches AER + ivpack", {
 
-  skip_if_not_installed("AER")
-  skip_if_not_installed("ivpack")
-  skip_if_not_installed("clubSandwich")
-
-  library(AER)
-  library(ivpack)
-
   ivco <- iv_robust(y ~ x | z, data = dat, se_type = "classical")
-  ivfit <- ivreg(y ~ x | z, data = dat)
+  ivfit <- AER::ivreg(y ~ x | z, data = dat)
   ivo <- summary(ivfit)
 
   expect_equivalent(
@@ -43,18 +36,28 @@ test_that("iv_robust matches AER + ivpack", {
   # Same as stata if you specify `small` as a stata option
   # which applies the N / N-k finite sample correction
 
+  expect_equivalent(
+    ivfit$fitted.values,
+    ivco$fitted.values
+  )
+
   # Stata defaults to HC0 as well, but does HC1 with `small`
   ivro <- iv_robust(y ~ x | z, data = dat, se_type = "HC0")
-  ivpackrob <- robust.se(ivfit)
+  capture_output(ivpackrob <- ivpack::robust.se(ivfit))
 
   expect_equivalent(
     as.matrix(tidy(ivro)[, c("estimate", "std.error", "p.value")]),
     ivpackrob[, c(1, 2, 4)]
   )
 
+  expect_equivalent(
+    ivfit$fitted.values,
+    ivro$fitted.values
+  )
+
   # "Stata" clustered SEs are CR0, but they are the same as below with `small`
   ivclusto <- iv_robust(y ~ x | z, data = dat, se_type = "stata", clusters = clust)
-  ivpackclust <- cluster.robust.se(ivfit, dat$clust)
+  capture_output(ivpackclust <- ivpack::cluster.robust.se(ivfit, dat$clust))
 
   # Our p-values are bigger (ivpack is be using less conservative DF, we use J - 1 which
   # is what stata uses for clusters w/ `small` and in OLS)
@@ -63,6 +66,11 @@ test_that("iv_robust matches AER + ivpack", {
     ivpackclust[, c(1, 2)]
   )
 
+  expect_equivalent(
+    ivfit$fitted.values,
+    ivclusto$fitted.values
+  )
+
   # CR2
   ivcr2o <- iv_robust(y ~ x | z, data = dat, clusters = clust)
   clubsando <- clubSandwich::coef_test(ivfit, vcov = "CR2", cluster = dat$clust)
@@ -71,6 +79,12 @@ test_that("iv_robust matches AER + ivpack", {
     as.matrix(tidy(ivcr2o)[, c("estimate", "std.error", "df", "p.value")]),
     as.matrix(clubsando)
   )
+
+  expect_equivalent(
+    ivfit$fitted.values,
+    ivcr2o$fitted.values
+  )
+
   # CR0
   ivcr0o <- iv_robust(y ~ x | z, data = dat, clusters = clust, se_type = "CR0")
   clubsandCR0o <- clubSandwich::coef_test(ivfit, vcov = "CR0", cluster = dat$clust, test = "naive-t")
@@ -82,7 +96,7 @@ test_that("iv_robust matches AER + ivpack", {
 
   # Weighting classical
   ivcw <- iv_robust(y ~ x | z, data = dat, weights = w, se_type = "classical")
-  ivw <- ivreg(y ~ x | z, weights = w, data = dat)
+  ivw <- AER::ivreg(y ~ x | z, weights = w, data = dat)
   ivregsum <- summary(ivw)
 
   expect_equivalent(
@@ -90,15 +104,25 @@ test_that("iv_robust matches AER + ivpack", {
     coef(ivregsum)[, c(1, 2, 4)]
   )
 
-  # HC1 weighted
+  expect_equivalent(
+    ivw$fitted.values,
+    ivcw$fitted.values
+  )
+
+  # HC0 weighted
   ivrw <- iv_robust(y ~ x | z, data = dat, weights = w, se_type = "HC0")
-  ivpackrobw <- robust.se(ivw)
+  capture_output(ivpackrobw <- ivpack::robust.se(ivw))
 
   expect_equivalent(
     as.matrix(tidy(ivrw)[, c("estimate", "std.error", "p.value")]),
     ivpackrobw[, c(1, 2, 4)]
   )
 
+  expect_equivalent(
+    ivrw$fitted.values,
+    ivcw$fitted.values
+  )
+
   # CR2 weighted
   ivcr2wo <- iv_robust(y ~ x | z, data = dat, clusters = clust, weights = w)
   clubsandwo <- clubSandwich::coef_test(ivw, vcov = "CR2", cluster = dat$clust)
@@ -108,6 +132,11 @@ test_that("iv_robust matches AER + ivpack", {
     as.matrix(clubsandwo)
   )
 
+  expect_equivalent(
+    ivcr2wo$fitted.values,
+    ivcw$fitted.values
+  )
+
   # CR0 weighted
   ivcr0wo <- iv_robust(y ~ x | z, data = dat, clusters = clust, weights = w, se_type = "CR0")
   clubsandCR0wo <- clubSandwich::coef_test(ivw, vcov = "CR0", cluster = dat$clust, test = "naive-t")
@@ -117,12 +146,17 @@ test_that("iv_robust matches AER + ivpack", {
     as.matrix(clubsandCR0wo)
   )
 
+  expect_equivalent(
+    ivcr0wo$fitted.values,
+    ivcw$fitted.values
+  )
+
   # Rank-deficiency
   # HC0
   dat$x1_c <- dat$x
   ivdefr <- iv_robust(y ~ x + x1_c| z + z2, data = dat, se_type = "HC0")
-  ivdef <- ivreg(y ~ x + x1_c| z + z2, data = dat)
-  ivdefse <- robust.se(ivdef)
+  ivdef <- AER::ivreg(y ~ x + x1_c| z + z2, data = dat)
+  capture_output(ivdefse <- ivpack::robust.se(ivdef))
 
   expect_equal(
     coef(ivdefr),
@@ -134,25 +168,31 @@ test_that("iv_robust matches AER + ivpack", {
     ivdefse[, c(1, 2, 4)]
   )
 
-  # Also works as instrument
-  ivdefri <- iv_robust(y ~ z + z2| x + x1_c, data = dat, se_type = "HC0")
-  ivdefi <- ivreg(y ~ z + z2| x + x1_c, data = dat)
-  ivdefsei <- robust.se(ivdefi)
-
-  expect_equal(
-    coef(ivdefri),
-    coef(ivdefi)
-  )
-
   expect_equivalent(
-    as.matrix(tidy(ivdefri)[1:2, c("estimate", "std.error", "p.value")]),
-    ivdefsei[, c(1, 2, 4)]
+    ivdefr$fitted.values,
+    ivdef$fitted.values
   )
 
+  # # Does not work if instrument is collinear with other instrument
+  # ivdefri <- iv_robust(y ~ z + z2| x + x1_c, data = dat, se_type = "HC0")
+  # ivdefi <- AER::ivreg(y ~ z + z2| x + x1_c, data = dat)
+  # ivdefsei <- ivpack::robust.se(ivdefi)
+  #
+  # # No longer equal!
+  # expect_equal(
+  #   coef(ivdefri),
+  #   coef(ivdefi)
+  # )
+
+  # expect_equivalent(
+  #   as.matrix(tidy(ivdefri)[1:2, c("estimate", "std.error", "p.value")]),
+  #   ivdefsei[, c(1, 2, 4)]
+  # )
+
   # Stata
   ivdefclr <- iv_robust(y ~ x + x1_c | z + z2, data = dat, clusters = clust, se_type = "stata")
-  ivdefcl <- ivreg(y ~ x + x1_c | z + z2, data = dat)
-  ivdefclse <- cluster.robust.se(ivdefcl, clusterid = dat$clust)
+  ivdefcl <- AER::ivreg(y ~ x + x1_c | z + z2, data = dat)
+  capture_output(ivdefclse <- ivpack::cluster.robust.se(ivdefcl, clusterid = dat$clust))
 
   expect_equal(
     coef(ivdefclr),
@@ -164,9 +204,15 @@ test_that("iv_robust matches AER + ivpack", {
     ivdefclse[, c(1, 2)]
   )
 
+
+  expect_equivalent(
+    ivdefclr$fitted.values,
+    ivdefcl$fitted.values
+  )
+
   # CR2
   ivdefcl2r <- iv_robust(y ~ x + x1_c | z + z2, data = dat, clusters = clust, se_type = "CR2")
-  ivdefcl2 <- ivreg(y ~ x + x1_c | z + z2, data = dat)
+  ivdefcl2 <- AER::ivreg(y ~ x + x1_c | z + z2, data = dat)
   ivdefcl2se <- clubSandwich::coef_test(ivdefcl2, vcov = "CR2", cluster = dat$clust)
 
 
@@ -175,10 +221,15 @@ test_that("iv_robust matches AER + ivpack", {
     na.omit(as.matrix(ivdefcl2se))
   )
 
+  expect_equivalent(
+    ivdefcl2r$fitted.values,
+    ivdefcl2$fitted.values
+  )
+
   # HC0 Weighted
   ivdefrw <- iv_robust(y ~ x + x1_c| z + z2, weights = w, data = dat, se_type = "HC0")
-  ivdefw <- ivreg(y ~ x + x1_c| z + z2, weights = w, data = dat)
-  ivdefsew <- robust.se(ivdefw)
+  ivdefw <- AER::ivreg(y ~ x + x1_c| z + z2, weights = w, data = dat)
+  capture_output(ivdefsew <- ivpack::robust.se(ivdefw))
 
   expect_equal(
     coef(ivdefrw),
@@ -190,9 +241,14 @@ test_that("iv_robust matches AER + ivpack", {
     ivdefsew[, c(1, 2, 4)]
   )
 
+  expect_equivalent(
+    ivdefrw$fitted.values,
+    ivdefw$fitted.values
+  )
+
   # CR2 Weighted
   ivdefclrw <- iv_robust(y ~ x + x1_c | z + z2, data = dat, clusters = clust, weights = w, se_type = "CR2")
-  ivdefclw <- ivreg(y ~ x + x1_c | z + z2, weights = w, data = dat)
+  ivdefclw <- AER::ivreg(y ~ x + x1_c | z + z2, weights = w, data = dat)
   ivdefclsew <- clubSandwich::coef_test(ivdefclw, vcov = "CR2", cluster = dat$clust)
 
   expect_equivalent(
@@ -200,27 +256,43 @@ test_that("iv_robust matches AER + ivpack", {
     na.omit(as.matrix(ivdefclsew)[, c(1, 2, 4)])
   )
 
+  expect_equivalent(
+    ivdefclrw$fitted.values,
+    ivdefclw$fitted.values
+  )
 
   ivdef2clrw <- iv_robust(y ~ x + z | x + x1_c, data = dat, clusters = clust, weights = w, se_type = "CR2")
-  ivdef2clw <- ivreg(y ~ x + z | x + x1_c, weights = w, data = dat)
+  ivdef2clw <- AER::ivreg(y ~ x + z | x + x1_c, weights = w, data = dat)
   ivdef2clsew <- clubSandwich::coef_test(ivdef2clw, vcov = "CR2", cluster = dat$clust)
 
   expect_equivalent(
     na.omit(as.matrix(tidy(ivdef2clrw)[, c("estimate", "std.error", "p.value")])),
     na.omit(as.matrix(ivdef2clsew)[, c(1, 2, 4)])
   )
+
+  expect_equivalent(
+    ivdef2clrw$fitted.values,
+    ivdef2clw$fitted.values
+  )
+
+  # F-stat fails properly with blocks of size 1
+  set.seed(42)
+  N <- 20
+  dat <- data.frame(y = rnorm(N), x = rnorm(N), z = rnorm(N), bl = sample(letters, size = N, replace = T))
+  ivr <- iv_robust(y ~ bl + x | bl + z, data = dat, se_type = "stata")
+  expect_equivalent(
+    ivr$fstatistic[1],
+    NA_integer_
+  )
+
 })
 
 
 test_that("iv_robust different specifications work", {
-  skip_if_not_installed("AER")
-  skip_if_not_installed("ivpack")
-  library(AER)
-
   # More instruments than endog. regressors
   ivro <- iv_robust(mpg ~ wt | hp + cyl, data = mtcars, se_type = "HC0")
-  ivo <- ivreg(mpg ~ wt | hp + cyl, data = mtcars)
-  ivpo <- robust.se(ivo)
+  ivo <- AER::ivreg(mpg ~ wt | hp + cyl, data = mtcars)
+  capture_output(ivpo <- ivpack::robust.se(ivo))
   expect_equivalent(
     as.matrix(tidy(ivro)[, c("estimate", "std.error", "p.value")]),
     ivpo[, c(1, 2, 4)]
@@ -228,8 +300,8 @@ test_that("iv_robust different specifications work", {
 
   # . notation for multiple exog, doesnt work!
   # ivro <- iv_robust(mpg ~ wt + hp + vs | . - vs + cyl, data = mtcars, se_type = "HC0")
-  # ivo <- ivreg(mpg ~ wt + hp + vs | . - vs + cyl, data = mtcars)
-  # ivpo <- robust.se(ivo)
+  # ivo <- AER::ivreg(mpg ~ wt + hp + vs | . - vs + cyl, data = mtcars)
+  # ivpo <- ivpack::robust.se(ivo)
   # expect_equivalent(
   #   as.matrix(tidy(ivro)[, c("estimate", "std.error", "p.value")]),
   #   ivpo[, c(1, 2, 4)]
@@ -237,8 +309,9 @@ test_that("iv_robust different specifications work", {
 
   # . notation in general
   ivro <- iv_robust(mpg ~ .| ., data = mtcars, se_type = "HC0")
-  ivo <- ivreg(mpg ~ . | ., data = mtcars)
-  ivpo <- robust.se(ivo)
+  ivo <- AER::ivreg(mpg ~ . | ., data = mtcars)
+  capture_output(ivpo <- ivpack::robust.se(ivo))
+
   expect_equivalent(
     as.matrix(tidy(ivro)[, c("estimate", "std.error", "p.value")]),
     ivpo[, c(1, 2, 4)]
@@ -248,8 +321,6 @@ test_that("iv_robust different specifications work", {
 
 test_that("S3 methods", {
 
-  skip_if_not_installed("AER")
-
   ivo <- AER::ivreg(mpg ~ hp + cyl | wt + gear, data = mtcars)
   ivro <- iv_robust(mpg ~ hp + cyl | wt + gear, data = mtcars, se_type = "classical")
 
@@ -316,3 +387,4 @@ test_that("S3 methods", {
   )
 
 })
+
